//#include "Riostream.h"
//#include <TApplication.h>
//#include <TString.h>
#include <TFile.h>
#include <TCanvas.h>
#include <TH2F.h>
#include <TPad.h>
#include <TView.h>
#include <TStyle.h>

#include <iostream>
#include <fstream>
#include <sstream>
//#include <deque>
#include <vector>
#include <map>
#include <cmath>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>

#include "util.h"

using std::string;
using std::cout;
using std::cerr;
using std::endl;
using std::map;
using std::vector;

string fmt = "png";

int runAnalysis(const char *inputFileName)
{
    struct stat st;
    stat(inputFileName, &st);
    time_t mtime = st.st_mtime;
    //    const string inputFileName;
    std::ifstream ifs(inputFileName);
    if (!ifs.good()) {
        cerr << "Can't open file " << inputFileName << endl;
        return 1;
    }

    // read text file
    vector<vector<double> > data;
    {
        vector<double> lineData;
        string str;
        int lineNumber = 1;
        while (ifs.good() && !ifs.eof() && std::getline(ifs, str)) {
            double d;
            lineData.clear();
            std::istringstream in(str);
            //        in.sync();
            while(in >> d) {
                lineData.push_back(d);
            }

            if (lineData.size() < 1) {
                cerr << lineNumber << ": syntax error: " << str << endl;
                break;
            }
            if (lineData[0] > 2.0) // upper frequency
                break;
            data.push_back(lineData);
            lineNumber++;
            if (lineNumber % 1000 == 0) {
                cout << "Read " <<  lineNumber << " lines" << endl;
            }
        }
        cout << "Read " <<  lineNumber << " lines" << endl;
    }

    vector<vector<double> > data3 = data;
    /*
    // average data, column 1 is value, other columns are dB
    const int num_average = 16;
    vector<vector<double> > data2(data.size()/num_average + 1);
    for(int i=0; i<data.size(); i++) {
	const int lineSize = data[i].size();
        if (lineSize < 1)
            break;
	const int j = i/num_average;
	data2[j].resize(lineSize);
	data2[j][0] += data[i][0]; // add as value
        for(int k=1; k<lineSize; k++) {
    	    data2[j][k] += exp(data[i][k]/20); // add as dB
        }
    }

    // convert to dB
    vector<vector<double> > data3(data2.size());
    for(int i=0; i<data2.size(); i++) {
	const int lineSize = data[i].size();
	data3[i].resize(lineSize);
        if (lineSize < 1)
            break;
        data3[i][0] = data2[i][0];
        for(int k=1; k<lineSize; k++)
            data3[i][k] = 20*log(data2[i][k]);
    }
*/

    if (data3.size() < 1)
	return 1;

    if (data3[0].size() < 4)
	return 1;

    const int ndev = (data3[0].size() - 1)/4; // number of devices
    const int nfft = data3.size(); // number of FFT bins
    const double dbrange = 115;
    const Int_t nx = nfft;
//    const Int_t ny = ndev;
    const Float_t fmin = data3[0][0];
    const Float_t fmax = data3[data3.size()-1][0];
//    const double dy = 0.0;
    const int nbinsy = ndev;
    const double ymin = 0.5;
    const double ymax = 0.5+nbinsy;
    cout << "Number of devices: " << ndev << endl;

    string hid[6];
    hid[0] = "h2sr";
    hid[1] = "h2dr";
    hid[2] = "h2sz";
    hid[3] = "h2dz";
    hid[4] = "h2srz";
    hid[5] = "h2drz";
    map<string, string> htitle;
    const string timeTag = string(" ") + timeStr(mtime);
    htitle["h2sr"] = string("FFT(Sum R)") + timeTag;
    htitle["h2sz"] = string("FFT(Sum Z)") + timeTag;
    htitle["h2srz"] = string("|FFT(Sum R - Sum Z)|") + timeTag;
    htitle["h2dr"] = string("FFT(Delta R)") + timeTag;
    htitle["h2dz"] = string("FFT(Delta Z)") + timeTag;
    htitle["h2drz"] = string("|FFT(Delta R - Delta Z)|") + timeTag;
    TH2F *hists[6];
    for (int i=0; i<6; i++) {
	hists[i] = new TH2F(hid[i].c_str(), htitle[hid[i]].c_str(), nx, fmin, fmax, nbinsy, ymin, ymax);
    }
/*
    TH2F *h2sr = new TH2F("h2sr", "Sum   R FFT" + timeStr(),             nx, fmin, fmax, nbinsy, ymin, ymax);
    TH2F *h2srz = new TH2F("h2srz", "Sum R - Sum Z FFT",     nx, fmin, fmax, nbinsy, ymin, ymax);
    TH2F *h2dr = new TH2F("h2dr", "Delta R FFT",             nx, fmin, fmax, nbinsy, ymin, ymax);
    TH2F *h2drz = new TH2F("h2drz", "Delta R - Delta Z FFT", nx, fmin, fmax, nbinsy, ymin, ymax);
    TH2F *h2sz = new TH2F("h2sz", "Sum   Z FFT",             nx, fmin, fmax, nbinsy, ymin, ymax);
    TH2F *h2dz = new TH2F("h2dz", "Delta Z FFT",             nx, fmin, fmax, nbinsy, ymin, ymax);
*/

//    const Float_t delta_gain = 2.4;
//    const Float_t delta_gain_db = 20*log(delta_gain)/log(10);

    for(size_t nf=0; nf<data3.size(); nf++) {
        const vector<double> &line = data3[nf];
//        double f = line[0];
        size_t pos = 1;
        int ch = 0;
        while(pos+3 < line.size()) {
            double v[4];
            v[0] = line[pos];
            v[1] = line[pos+1];
            v[2] = line[pos+2];
            v[3] = line[pos+3];
            for(int i=0; i<4; i++) {
                if (v[i] < -dbrange) v[i] = -dbrange;
                v[i] += dbrange;
                //                if ((i%2) == 1) {v[i] -= delta_gain_db;}
                //                v[i] /= num_average;
            }
            hists[0]->SetBinContent(1+nf, 1+ch, v[0]);
            hists[1]->SetBinContent(1+nf, 1+ch, v[1]);
            hists[2]->SetBinContent(1+nf, 1+ch, v[2]);
            hists[3]->SetBinContent(1+nf, 1+ch, v[3]);
            hists[4]->SetBinContent(1+nf, 1+ch, fabs(v[0]-v[2]));
            hists[5]->SetBinContent(1+nf, 1+ch, fabs(v[1]-v[3]));
            pos += 4;
            ch ++;
        }
    }

    TString s = "Lego2";
    gStyle->SetPalette (1, 0);
    gStyle->SetOptStat(0);
    gStyle->SetPaperSize(12, 9);
    const TString titleX = "Frequency, MHz";
    const TString titleY = "Pickup number";
    const TString titleZ = "dB";

    string imageFileName[6];
    string imageFileNameTmp[6];
    for(int i=0; i<6; i++) {
        TCanvas *c = new TCanvas("c", "c", 620, 512);
//        TCanvas *c = new TCanvas("c", "c", 480, 380);
        TH2F *h = hists[i];
        h->Draw(s);
        TVirtualPad *p = c->Pad();
        h->GetXaxis()->SetTitle(titleX);
        h->GetYaxis()->SetTitle(titleY);
        h->GetZaxis()->SetTitle(titleZ);
        p->Update();
        p->GetView()->RotateView(-30, 61);
        imageFileName[i] = string("bpm-")+hid[i] + string(".")+fmt;
        imageFileNameTmp[i] = string("bpm-")+hid[i] + string(".tmp.")+fmt;
        p->Print(imageFileNameTmp[i].c_str(), fmt.c_str());
        delete c;
    }
    for (int i=0; i<6; i++) {
	unlink(imageFileName[i].c_str());
	rename(imageFileNameTmp[i].c_str(), imageFileName[i].c_str());
    }
    /*
//    c1->Divide(3,2);
//    c1->SetMargin(0, 0, 0, 0);

    c1->cd(1);
    h2sr->Draw(s);
    c1->cd(2);
    h2sz->Draw(s);
    c1->cd(3);
    h2srz->Draw(s);

    c1->cd(4);
    h2dr->Draw(s);
    c1->cd(5);
    h2dz->Draw(s);
    c1->cd(6);
    h2drz->Draw(s);

    c1->Update();

    for(int i=0; i<6; i++) {
	TPad *pad = c1->GetPad(1+i);
	pad->cd();
//	if (!pad) break;
	TView *view = pad->GetView();
//	if (!view) break;
	view->RotateView(-30, 61);
    }
    c1->Modified();
    c1->Update();
*/
    TFile *file = new TFile("bpm.tmp.root","RECREATE");
    for (int i=0; i<6; i++) {
	hists[i]->Write();
    }
    file->Close();
    delete file;
    for (int i=0; i<6; i++) {
	delete hists[i];
    }
    unlink("bpm.root");
    rename("bpm.tmp.root", "bpm.root");
    return 0;
}

void bpm()
{
    runAnalysis("fft_all.txt");
}

int runAnalysisLoop(const char *inputFileName)
{
    time_t last_mtime = 0;
    while(true) {
        struct stat st;
        if (0 == stat(inputFileName, &st)) {
    	    if (st.st_mtime != last_mtime) {
		last_mtime = st.st_mtime;
		runAnalysis(inputFileName);
	    }
	}
	usleep(1000000);
    }
}

int main(int argc, char *argv[])
{
    if (argc < 2) {
        cerr << "USAGE: " << argv[0] << " <filename> [gif|png|svg]" << endl;
        return 1;
    }
    if (argc == 3) {
        string fmtarg = argv[2];
        if (fmtarg == "gif" || fmtarg == "png" || fmtarg == "svg")
    	    fmt = fmtarg;
    	else
    	    cerr << "Unsupported image format " << fmtarg << ", using default" << endl;
    }
    runAnalysisLoop(argv[1]);
    //    TApplication app("app", &argc, argv);
    //    app.Run();
    return 0;
}
