//
//    Copyright 2012 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

//#include "Riostream.h"
#include <TROOT.h>
#include <TApplication.h>
#include <TBrowser.h>
//#include <TString.h>
#include <TFile.h>
#include <TCanvas.h>
#include <TGraph.h>
#include <TH1F.h>
#include <TH2F.h>
#include <TPad.h>
#include <TView.h>
#include <TStyle.h>
#include <TAxis.h>

#include <iostream>
#include <fstream>
#include <sstream>
#include <deque>
#include <vector>
#include <map>
//#include <set>
#include <cmath>
#include <locale>
#include <cassert>
#include <algorithm>

#include <QMap>
#include <QDebug>
#include <QDateTime>
#include <QFileInfo>
#include <QTextCodec>

//#include <time.h>
#include <wchar.h>

#include "dsplib/dsplib.h"
#include "dsplib/fftengine.h"

using std::string;
using std::wstring;
using std::cout;
using std::wcout;
using std::cerr;
using std::wcerr;
using std::endl;
using std::map;
using std::vector;
using std::deque;
using std::copy;

const int fileStatPeriodUs = 200000; // 0.2s

QString fileBase = "field";
QString fmt = "png";

//fwinf_t winfunc = WF_BLACKMANHARRIS;
//fwinf_t winfunc = WF_HANN;
//fwinf_t winfunc = WF_BLACKMAN;
fwinf_t winfunc = WF_HAMMING;
//fwinf_t winfunc = WF_BARLETT;
//--------------------------------------------------------------

vector<double> applyWindow(const vector<double> &x, fwinf_t winfunc)
{
    int N = x.size();
    vector<double> z(N);
    for(int i=0; i<N; i++)
        z[i] = x[i] * windowingFunction(winfunc, i, N); // * 1. / wind_norm;
    return z;
}

class MyFFTEngine : public RFFTEngine {
    public:
    vector<double> getPowerSpectrum(const vector<double> &d);
};

vector<double> MyFFTEngine::getPowerSpectrum(const vector<double> &d)
{
    if (d.size() != N) {
        resize(d.size());
        //	throw exc("Incompatible input data size for FFT");
    }
    const size_t nre = N/2+1;
    for(size_t i=0; i<N; i++)
        plan.in[i] = d[i] * windowingFunction(winfunc, i, N) / wind_norm;
    rfftw_one(plan.fftwr_p, plan.in, plan.out);
    vector<double> result;
    result.resize(nre);
    // out is in halfcomplex format:
    // 0<=i<=N/2, out[i]=Re(Xi)
    // 0<i<N/2,   out[N-i]=Im(Xi)
    const fftw_real *out = plan.out;;
    result[0] = out[0]*out[0];  /* DC component */
    for (size_t k = 1; k < (N+1)/2; ++k) { /* (k < N/2 rounded up) */
        result[k] = out[k]*out[k] + out[N-k]*out[N-k];
    }
    if (N % 2 == 0) { /* N is even */
        result[N/2] = out[N/2]*out[N/2];  /* Nyquist freq. */
    }

    for(size_t i=0; i<nre; i++) {
        result[i] = sqrt(result[i]) / N;
    }
    return result;
}


vector<double> extend(const vector<double> &x, int padsize)
{
    if (x.empty()) return x;
    vector<double> z(x.size() + 2*padsize);
    double xhead = x[0];
    double xtail = x[x.size()-1];
    xhead = 0; xtail = 0; // Zero-padding
    fill(z.begin(), z.begin()+padsize, xhead);
    copy(x.begin(), x.end(), z.begin()+padsize);
    fill(z.begin()+padsize+x.size(), z.end(), xtail);
/*    for(int i=0; i<z.size(); i++)
       cout << z[i] << endl;
    exit(0);
*/    return z;
}

/*
// 2D Waterfall on real data
vector<vector<double> > makeWaterfall(const vector<double> &x, int fftBins, double overlap)
{
    vector<vector<double> > wf;
    if (x.empty()) return wf;
    vector<double> ext = extend(x, fftBins/2);
//    vector<double> ext = x;
    MyFFTEngine fft;
    double pos = 0;
    double step = fftBins * overlap;
    while (pos + fftBins <= ext.size()) {
        // get slice
        vector<double> slice(fftBins);
        copy(&ext[pos], &ext[pos+fftBins], slice.begin());
        // perform fft with window function
//        slice = applyWindow(slice, winfunc);
//        slice = extend(slice, slice.size()/2);
        vector<double> spectrum = fft.getPowerSpectrum(slice);
        spectrum.resize(spectrum.size() / downsampleFactor);
        pos += step;
        wf.push_back(spectrum);
    }
//    cerr << "wf: " << wf.size() << endl;
    return wf;
}
*/

void visualizeSampleData(const vector<double> &samples)
{
    double signalLevelAbsoluteMinimumDb = -150;
    double adcFreq = 1e3; // 1 kHz

    if (samples.size() < 4) return;
    MyFFTEngine fft;
    vector<double> py = fft.getPowerSpectrum(samples);
    double fftMaxFreq = adcFreq/2; // value in Hz
    int n = py.size();
    vector<double> px(n);

    for(int i=0; i<n; i++) {
        px[i] = fftMaxFreq / n * i;
        py[i] = 20 * log(py[i]) / log(10);
        if (py[i] < signalLevelAbsoluteMinimumDb) py[i] = signalLevelAbsoluteMinimumDb; // limit
    }

    double x1 = px[0];
    double x2 = px[n-1];
    double y1 = signalLevelAbsoluteMinimumDb;
    double y2 = 60;
    TCanvas *c = new TCanvas("c1", "c1", 480, 320);
    c->SetGrid();
    c->cd();
    TH1F *h = c->DrawFrame(x1, y1, x2, y2);
    assert(h);
    TGraph *gr = new TGraph(n, &px[0], &py[0]);
    gr->Draw("LP");
    h->GetYaxis()->SetTitle("dB");
    h->GetXaxis()->SetTitle("Frequency, Hz");
    h->SetTitle("Field Spectrum");
    {
        TVirtualPad *p = c->Pad();
        assert(p);
        assert(p->GetFrame());
        p->Update();
        QString imageFileName = fileBase + QString(".")+fmt;
        QString imageFileNameTmp = fileBase + QString(".tmp.")+fmt;
        p->Print(imageFileNameTmp.toLocal8Bit(), fmt.toLocal8Bit());
        unlink(imageFileName.toLocal8Bit());
        rename(imageFileNameTmp.toLocal8Bit(), imageFileName.toLocal8Bit());
        qDebug() << QString("%1 printed ok").arg(imageFileName);
        delete h;
        delete c;
    }
}

int runAnalysis(const QString &inputFileName)
{
    vector<double> data;
    ifstream in;
    in.open(inputFileName.toStdString().c_str());
    while (true) {
        double d;
        in >> d;
        if (!in.good()) break;
        data.push_back(d);
    }
    visualizeSampleData(data);
    return 0;
}

int runAnalysisLoop(const char *inputFileName)
{
    QDateTime last_mtime;
    while(true) {
        QDateTime mtime = QFileInfo(inputFileName).lastModified();
        if (mtime != last_mtime) {
            last_mtime = mtime;
            runAnalysis(QString::fromLocal8Bit(inputFileName));
        }
        usleep(fileStatPeriodUs);
    }
    return 0;
}

void rootSucks()
{
    gROOT->SetStyle("Plain");
    gStyle->SetPalette (1, 0);
    gStyle->SetOptStat(0);
    gStyle->SetTitleAlign(23); // holy shit
    gStyle->SetTitleX(0.5);
    gStyle->SetTitleBorderSize(0);
    gStyle->SetTitleFontSize(0.06);
}

int main(int argc, char *argv[])
{
    std::locale::global(std::locale(""));
    QTextCodec::setCodecForCStrings(QTextCodec::codecForName("utf8"));
    if (argc < 2) {
        qFatal(QString("USAGE: %1 <filename.dat>")
               .arg(QString::fromLocal8Bit(argv[0])).toLocal8Bit());
        return 1;
    }

    QString fileName = QString::fromLocal8Bit(argv[1]);
    rootSucks();

    if (0 != runAnalysis(fileName)) {
        qCritical() << "Failed to start";
        return 1;
    }
    runAnalysisLoop(fileName.toLocal8Bit());
    return 0;
}
