//
//    Copyright 2012 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

//#include "Riostream.h"
#include <TROOT.h>
#include <TApplication.h>
#include <TBrowser.h>
//#include <TString.h>
#include <TFile.h>
#include <TCanvas.h>
#include <TGraph.h>
#include <TH1F.h>
#include <TH2F.h>
#include <TPad.h>
#include <TView.h>
#include <TStyle.h>
#include <TAxis.h>

#include <iostream>
#include <fstream>
#include <sstream>
#include <deque>
#include <vector>
#include <map>
//#include <set>
#include <cmath>
#include <locale>
#include <cassert>
#include <algorithm>

#include <QMap>
#include <QDebug>
#include <QDateTime>
#include <QFileInfo>

//#include <time.h>
#include <wchar.h>

#include "bpmdata.h"
//#include "bpmconst.h"
#include "bpmlib/bpmevent.h"
#include "dsplib/dsplib.h"
#include "dsplib/fftengine.h"

using std::string;
using std::wstring;
using std::cout;
using std::wcout;
using std::cerr;
using std::wcerr;
using std::endl;
using std::map;
using std::vector;
using std::deque;
using std::copy;

const int fileStatPeriodUs = 200000; // 0.2s

QString fileBase = "wf";
QString fmt = "png";
int selectChannel = 1; // 0..3, -1 for all (batch mode)
int selectId = -1;
//--------------------------------------------------------------
bool makeplots = false;
int    fft_bins = 1024*32;
double fft_overlap = 0.125/8;
double adcFreq = 100e6;
int downsampleFactor = 1;//32;

fwinf_t winfunc = WF_BLACKMANHARRIS;
//fwinf_t winfunc = WF_HANN;
//fwinf_t winfunc = WF_BLACKMAN;
//fwinf_t winfunc = WF_HAMMING;
//fwinf_t winfunc = WF_BARLETT;
//--------------------------------------------------------------

vector<double> applyWindow(const vector<double> &x, fwinf_t winfunc)
{
    int N = x.size();
    vector<double> z(N);
    for(size_t i=0; i<N; i++)
        z[i] = x[i] * windowingFunction(winfunc, i, N); // * 1. / wind_norm;
    return z;
}

class MyFFTEngine : public RFFTEngine {
    public:
    vector<double> getPowerSpectrum(const vector<double> &d);
};

vector<double> MyFFTEngine::getPowerSpectrum(const vector<double> &d)
{
    if (d.size() != N) {
        resize(d.size());
        //	throw exc("Incompatible input data size for FFT");
    }
    const size_t nre = N/2+1;
    for(size_t i=0; i<N; i++)
        plan.in[i] = d[i] * windowingFunction(winfunc, i, N) / wind_norm;
    rfftw_one(plan.fftwr_p, plan.in, plan.out);
    vector<double> result;
    result.resize(nre);
    // out is in halfcomplex format:
    // 0<=i<=N/2, out[i]=Re(Xi)
    // 0<i<N/2,   out[N-i]=Im(Xi)
    const fftw_real *out = plan.out;;
    result[0] = out[0]*out[0];  /* DC component */
    for (size_t k = 1; k < (N+1)/2; ++k) { /* (k < N/2 rounded up) */
        result[k] = out[k]*out[k] + out[N-k]*out[N-k];
    }
    if (N % 2 == 0) { /* N is even */
        result[N/2] = out[N/2]*out[N/2];  /* Nyquist freq. */
    }

    for(size_t i=0; i<nre; i++) {
        result[i] = sqrt(result[i]) / N;
    }
    return result;
}


vector<double> extend(const vector<double> &x, int padsize)
{
    if (x.empty()) return x;
    vector<double> z(x.size() + 2*padsize);
    double xhead = x[0];
    double xtail = x[x.size()-1];
    xhead = 0; xtail = 0; // Zero-padding
    fill(z.begin(), z.begin()+padsize, xhead);
    copy(x.begin(), x.end(), z.begin()+padsize);
    fill(z.begin()+padsize+x.size(), z.end(), xtail);
/*    for(int i=0; i<z.size(); i++)
       cout << z[i] << endl;
    exit(0);
*/    return z;
}

// 2D Waterfall on real data
vector<vector<double> > makeWaterfall(const vector<double> &x, int fftBins, double overlap)
{
    vector<vector<double> > wf;
    if (x.empty()) return wf;
    vector<double> ext = extend(x, fftBins/2);
//    vector<double> ext = x;
    MyFFTEngine fft;
    double pos = 0;
    double step = fftBins * overlap;
    while (pos + fftBins <= ext.size()) {
        // get slice
        vector<double> slice(fftBins);
        copy(&ext[pos], &ext[pos+fftBins], slice.begin());
        // perform fft with window function
//        slice = applyWindow(slice, winfunc);
//        slice = extend(slice, slice.size()/2);
        vector<double> spectrum = fft.getPowerSpectrum(slice);
        spectrum.resize(spectrum.size() / downsampleFactor);
        pos += step;
        wf.push_back(spectrum);
    }
//    cerr << "wf: " << wf.size() << endl;
    return wf;
}

void visualizeSampleData(const DaqAdcSamples &samples, double samplingFreq, QString title)
{
    const vector<vector<double> > &wf = makeWaterfall(samples.toStdVector(), fft_bins, fft_overlap);
    if (wf.empty()) return;
    if (wf[0].empty()) return;
    int nx = wf.size();
    int ny = wf[0].size();

    TCanvas *c = new TCanvas("c1", "c1", 1024, 1024);
    c->cd();
    double fftFreqScale = adcFreq/2/1e6 / downsampleFactor; // value in MHz
    double timeStep = fft_bins * fft_overlap / (adcFreq/1e3); // value in ms
    double yscale = (fftFreqScale / ny);
    double xscale = (timeStep);
    cerr << "time step: " << timeStep << " us"<< endl;
//    cerr << "xscale: " << xscale << endl;
//    cerr << "yscale: " << yscale << endl;
//    yscale=2;
//    double ymax = ny*yscale;
    TH2F *h = new TH2F("h2", "h2", nx, 0, nx*xscale, ny, 0, ny*yscale);
    h->SetStats(false);
    assert(h);
    for(int x=0; x<nx; x++)
        for(int y=0; y<ny; y++) {
            double v = wf.at(x).at(y) + 0.02;
            v = 20*log(v)/log(10);
//            h->Fill(x, y*yscale, v);
            h->SetBinContent(x, y, v);
    }
    h->SetContour(64);
    h->Draw("COLZ");
    h->GetXaxis()->SetTitle("Time, ms");
    h->GetYaxis()->SetTitle("Frequency, MHz");
//    h->GetYaxis()->SetRange(0., 0.025*ny);
    h->SetTitle(QString("BPM data %1").arg(title).toStdString().c_str());
    if (makeplots) {
        TVirtualPad *p = c->Pad();
        assert(p);
        assert(p->GetFrame());
        p->Update();
        QString imageFileName = fileBase + "-" + title + QString(".")+fmt;
        QString imageFileNameTmp = fileBase + "-" + title + QString(".tmp.")+fmt;
        p->Print(imageFileNameTmp.toLocal8Bit(), fmt.toLocal8Bit());
        unlink(imageFileName.toLocal8Bit());
        rename(imageFileNameTmp.toLocal8Bit(), imageFileName.toLocal8Bit());
        qDebug() << QString("%1 printed ok").arg(title);
    //    TFile rootFile("wf.root", "RECREATE");
    //    h->Write();
    //    rootFile.Close();
        delete h;
        delete c;
    }
}

int runAnalysis(const QString &inputFileName)
{
    const DaqEvents &daqEvents = DaqEvents::readEvents(inputFileName);
    cerr << "Read " << daqEvents.size() << " events" << endl;
    if (daqEvents.empty())
        return 0;
//    for(int ev=0; ev<daqEvents.size(); ev++) {
//    }
    int selectEvent = 0;
//    if (daqEvents.size() >= 1)
        selectEvent = daqEvents.size()/2;
    const DaqEvent &daqEvent = daqEvents.at(selectEvent);
    const DaqEventBoxes &daqEventBoxes = daqEvent.box;
    if (daqEventBoxes.empty())
        return 0;
    qDebug() << "Available devices:" << hex << daqEventBoxes.keys();
    if (selectId != -1) {
        if (!daqEventBoxes.keys().contains(selectId)) {
            selectId = daqEventBoxes.keys().first();
            qWarning() << QString("Selected ID not found, using master ID");
        }
    }
    Q_FOREACH(int id, daqEventBoxes.keys()) {
        if ((selectId != -1) && selectId != id) continue;
        const DaqEventBox &daqEventBox = daqEventBoxes[id];
        const DaqEventBoxAdcs &daqEventBoxAdcs = daqEventBox.getAdc();
        if (daqEventBoxAdcs.size() < 4)
            return 0;

        for(int n=0; n<4; n++) {
            if ((selectChannel != -1) && (n != selectChannel)) continue;
            const DaqEventBoxAdc &daqEventBoxAdc = daqEventBoxAdcs[n]; // 1:delta R
        //    const size_t numSamples = daqEventBoxAdc.size();
            const DaqAdcSamples &samples = daqEventBoxAdc.getSamples();
            if (samples.empty())
                continue;
            const double samplingFreq = daqEventBox.getSamplingFreq();
            QString title = QString("%1-%2").arg(id, 6, 16, QChar('0')).arg(n);
            visualizeSampleData(samples, samplingFreq, title);
        }
    }
    return 0;
}
/*
int runAnalysisLoop(const char *inputFileName)
{
    QDateTime last_mtime;
    while(true) {
        QDateTime mtime = QFileInfo(inputFileName).lastModified();
        if (mtime != last_mtime) {
            last_mtime = mtime;
            runAnalysis(QString::fromLocal8Bit(inputFileName));
        }
        usleep(fileStatPeriodUs);
    }
    return 0;
}
*/

void rootSucks()
{
    gROOT->SetStyle("Plain");
    gStyle->SetPalette (1, 0);
    gStyle->SetOptStat(0);
    gStyle->SetTitleAlign(23); // holy shit
    gStyle->SetTitleX(0.5);
    gStyle->SetTitleBorderSize(0);
    gStyle->SetTitleFontSize(0.06);
}

int main(int argc, char *argv[])
{
    std::locale::global(std::locale(""));
    if (argc < 2) {
        qFatal(QString("USAGE: %1 <filename.dat> [gif|png|svg]")
               .arg(QString::fromLocal8Bit(argv[0])).toLocal8Bit());
        return 1;
    }
    bool channelSelected = false;
    bool idSelected = false;
    for(int i=3; i<=argc; i++) {
        QString arg = argv[i-1];
        if (arg == "gif" || arg == "png" || arg == "svg") {
            fmt = arg;
            makeplots = true;
        }
        bool ok = false;
        QString str(arg);
        int n = str.toInt(&ok);
        if (ok && (n >= -1) && (n <= 3)) {
//            qDebug() << "Selected channel " << n;
            selectChannel = n;
            channelSelected = true;
        } else {
            quint64 id = str.toULongLong(&ok, 16);
            if (ok) {
//                qDebug() << QString("Selected ID %1").arg(id, 6, 16, QChar('0'));
                selectId = id;
                idSelected = true;
            }
        }
    }
    if (!channelSelected && makeplots) selectChannel = -1; // plot all channels in batch mode
    if (!idSelected && !makeplots) selectId = 0; // use master device by default in interactive

    QString fileName = QString::fromLocal8Bit(argv[1]);
    rootSucks();
    TApplication *app = 0;
    if (!makeplots) app = new TApplication("app", &argc, argv);
    if (makeplots) qDebug() << "Batch mode";
    if (selectId != -1)
        qDebug() << QString("Selected ID %1").arg(selectId, 6, 16, QChar('0'));
    if (channelSelected)
        qDebug() << QString("Selected channel %1").arg(selectChannel);

    if (0 != runAnalysis(fileName)) {
        qCritical() << "Failed to start";
        return 1;
    }
//    runAnalysisLoop(fileName);
    if (!makeplots) app->Run();
    return 0;
}
