//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef BPMDATA_H
#define BPMDATA_H

#include <string>
#include <map>

struct BpmEntry
{
    std::wstring label;
    std::wstring place;
    int event;
    float field;
    float pos;
    float posR, powerR, posZ, powerZ;
    float bgrR, bgrZ;
};

typedef std::map<std::wstring, BpmEntry> BpmEvent;
typedef std::map<int, BpmEvent> BpmRun;

class BpmSignalLevels: public std::map<int, double>
{
public:
    double min() const;
    double max() const;
protected:
};

class BpmData
{
public:
    BpmData(const std::string &inputFileName);
    static BpmRun getReadOrbitaCsv(const std::string &inputFileName);
    static BpmSignalLevels getEventSignalLevels(const BpmRun &bpmRun);

protected:
    std::string fileName;
};

bool BpmSignalLevel_value_comp(const std::pair<int, double>& lhs,
                               const std::pair<int, double>& rhs);


#endif // BPMDATA_H
