//
//    Beam Position Monitor
//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef BEAMPOSITION_H
#define BEAMPOSITION_H

#include <QMap>
#include <QVector>
#include <QMetaType>

#include <ostream>

class BeamPosition {
public:
    BeamPosition() { id=0; time = 0; B_Gs = 0; posH = 0; posV = 0; powerErrH = 0; powerErrV = 0; powerH = 0; powerV = 0; rmsH = 0; rmsV = 0; normFactor = 8192.;}
    friend std::ostream& operator<<(std::ostream &s, const BeamPosition &);
    int id;
    double time;
    double B_Gs;
    double posH;
    double posV;
    double powerH;
    double powerV;
    double powerHdB() const;
    double powerVdB() const;
    double powerErrH;
    double powerErrV;
    double powerErrHdB() const;
    double powerErrVdB() const;
    double normFactor;
    double rmsH;
    double rmsV;
};

class BeamPositionVector: public QVector<BeamPosition>
{
public:
    friend std::ostream& operator<<(std::ostream &s, const BeamPositionVector &);
};

class BeamPositions: public QMap<int,  BeamPositionVector> {
public:
    friend std::ostream& operator<<(std::ostream &s, const BeamPositions &);
};

Q_DECLARE_METATYPE(BeamPositions);

#endif // BEAMPOSITION_H
