//
//    Beam Position Monitor
//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#ifndef BPMDATATYPES_H
#define BPMDATATYPES_H

#include <QMap>
#include <QList>
#include <QVector>
#include <QDateTime>
#include <QMetaType>
#include "bpmlib/BeamPosition.h"
#include "dsplib/FFTSpectrum.h"

class AdcFFTSpectra: public QMap<int, FFTSpectrum> {};

class BoxAdcFFTSpectra: public QMap<int, AdcFFTSpectra> {
public:
    //	BoxAdcFFTSpectra& operator/(double n) const;
    //	BoxAdcFFTSpectra& operator+=(const BoxAdcFFTSpectra &);
};
Q_DECLARE_METATYPE(BoxAdcFFTSpectra);

class FreqMapEntry {
public:
    double time;
    double field;
    double freq;
    double freqVary;
};

class FreqMapChannel: public QVector<FreqMapEntry> {};
class FreqMap: public QMap<int, FreqMapChannel> {};

class DeviceClockFreqFactors: public QMap<int, double> { };
//class DeviceTemperatures: public QMap<int, double> { };
class DeviceAttr {
public:
    DeviceAttr();
    QString getFirmwareVersionStr() const;
    bool operator==(const DeviceAttr &) const;
    bool operator!=(const DeviceAttr &) const;

    int device_id;
    int device_serial;
    double temp;
    double clock;
    int fw_ver;
    int fw_rev;
};

class DeviceAttrs: public QMap<int, DeviceAttr> { };
Q_DECLARE_METATYPE(DeviceAttrs);

class DeviceIds: public QList<int> { };
Q_DECLARE_METATYPE(DeviceIds);

class BpmLinkStatus {
public:
    bool isUp() const { return !no_clock && !disconnect && !lcf_timeout && !remote_error;}
    bool no_clock;
    bool disconnect;
    bool lcf_timeout;
    bool remote_error;
    bool operator==(const BpmLinkStatus &rhs) const;
};

class BpmBoxLinkStatus {
public:
    BpmLinkStatus linkTx;
    BpmLinkStatus linkRx;
    BpmLinkStatus linkOpt;
    bool operator==(const BpmBoxLinkStatus &rhs) const;
};

class BpmLinkStatistics {
public:
    quint32 link_errors;
    quint32 crc_errors;
    bool operator==(const BpmLinkStatistics &rhs) const;
    bool operator!=(const BpmLinkStatistics &rhs) const;
};

class BpmBoxLinkStatistics {
public:
    BpmLinkStatistics linkTx;
    BpmLinkStatistics linkRx;
    BpmLinkStatistics linkOpt;
    bool operator==(const BpmBoxLinkStatistics &rhs) const;
    bool operator!=(const BpmBoxLinkStatistics &rhs) const;
};

Q_DECLARE_METATYPE(BpmBoxLinkStatistics);

class DeviceLinkStatistics: public QMap<int, BpmBoxLinkStatistics> { };

Q_DECLARE_METATYPE(DeviceLinkStatistics);

class BpmRawInfo {
public:
    qint32 b;
    qint64 ts_cur;
    qint64 ts_ncu;
    qint64 ts_kcu;
};

class BpmOnlineValues {
public:
    BpmOnlineValues();
    bool operator==(const BpmOnlineValues &) const;
    bool operator!=(const BpmOnlineValues &) const;
    QDateTime dateTime;
    double currentCycleTime;
    double currentBField;
    double lastBMax;
};

Q_DECLARE_METATYPE(BpmOnlineValues);

void registerBpmDataMetaTypes();

#endif // BPMDATATYPES_H
