//
//    Beam Position Monitor
//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <cmath>

#include "beamsetup.h"

const double PI = 3.1415926535897932384626433832795;
const double c = 2.99792458e8;

// electron charge
//double e = 1.6021892e-19;

double BeamSetup::b2freq(double B) const
{
    if (B<=0) B=0.0001;
    double d = (B*rho*c)/W0;
    double beta = d/sqrt(1+pow(d, 2));
    double f = kf*c/circumference * beta;
    //  printf("B=%e, rho=%e, d=%e, beta=%e, f=%e\n", B, rho, d, beta, f);
    return f;
}

double BeamSetup::freq2R(double B, double f) const
{
    if (f<0) return 0;
    double d = (B*rho*c)/W0;
    double beta = d/sqrt(1+pow(d, 2));
    double L = beta*kf*c/f;
    return L/(2*PI);
}

double BeamSetup::freq2deltaR(double B, double f) const
{
    double R = freq2R(B, f);
    double deltaR = R-circumference/(2*PI);
    return deltaR;
}

BeamSetup::BeamSetup()
{
    kf = 5;
    W0 = d_W0;
    rho = 22.0;
    circumference = 251.52;
}
