//
//    Beam Position Monitor
//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef BPMAPPSETTINGS_H
#define BPMAPPSETTINGS_H

#include "misclib/appsettings.h"
//#include "bpmlib/bpmbox.h"

typedef enum {
    BPMStartB0=0,
    BPMStartSoftware=1,
    BPMStartExt=2,
} start_select_t;

typedef enum {
    BPMTriggerTableBField=0,
    BPMTriggerTableTime=1,
    BPMTriggerPeriodic=2,
    BPMTriggerExt=3,
} trigger_select_t;

class StartSelect {
public:
    static start_select_t fromString(const QString &);
    static start_select_t fromVariant(const QVariant &);
    static QString toString(const start_select_t &);
};

class TriggerSelect {
public:
    static trigger_select_t fromString(const QString &);
    static trigger_select_t fromVariant(const QVariant &);
    static QString toString(const trigger_select_t &);
};

// Global limits
const double B0Min = 0.0;
const double B0Max = 20000.0; // Gs

const double BStepMin = 0.001; // 0.001 Gs
const double BStepMax = 1.0; // 1Gs

class BpmAppSettings: public AppSettings
{
public:
    BpmAppSettings( const QString & organization, const QString & application = QString());
    void setDefaults();
    void read(const QString &fileName = QString());
    void write(const QString &fileName = QString());
    QList<quint32> getSampleSizeList() const;
protected:
    static QList<QVariant> translateStringListToVariantList(const QStringList &sl);
    static QStringList translateVariantListToStringList(const QVariant &var);
private:
    QString _organization;
    QString _application;
};

#endif /* BPMAPPSETTINGS_H */
