//
//    Beam Position Monitor
//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef _BPM_EVENT_ANALYSER
#define _BPM_EVENT_ANALYSER

#include "dsplib/FFTSpectrum.h"
#ifdef USE_FITTER
#include "dsplib/fitter.h"
#endif
//#include "wfdata.h"
//#include "dsplib/myvector.h"
#include "beamsetup.h"

#include "bpmlib/BpmDataTypes.h"

class DaqEvent;
class DaqEvents;

/*
class FftPowDBFS {
  public:
        FftPowDBFS() {n=0;}
        myvector<double> v;
        size_t n;
};
*/
/*
class BoxAdcFFTSpectra {
  public:
        FFTSpectrum& operator+(const FFTSpectrum &);

  protected:
        FFTSpectrum spectrum;
        size_t count;
};
*/

class RFFTEngine;
#ifdef USE_FITTER
class FitterSine;
#endif

typedef QMap<int, quint64> BoxFftSpectraCount;
typedef QMap<int, BoxFftSpectraCount > FftSpectraCount;

class BpmEventAnalyser {
public:
    BpmEventAnalyser(const BeamSetup &);
    ~BpmEventAnalyser();
    void clearMeasurements();
    void clearSpectraSum();
    void addEvent(const DaqEvent &);
    void addEvents(const DaqEvents &);
    //	WaveformDataAll getFFTWaveforms() const;
    BoxAdcFFTSpectra getFFTSpectra() const;
    BeamPositions getBeamPositions() const { return beamPositions; }
#ifdef USE_FITTER
    FreqMap measureFrequency(const DaqEvents &events) const;
#endif
    void saveFieldFreqMap(const FreqMap &m, const QString &fileName) const;

protected:
    BoxAdcFFTSpectra fftSpectraSum;
    FftSpectraCount fftSpectraCount;
    const BeamSetup &beamSetup;
    BeamPositions beamPositions;
//    QMap<double, double> filedFreqMap;
    RFFTEngine *fft;
#ifdef USE_FITTER
    mutable FitterSine *fitter;
#endif
};

#endif /* _BPM_EVENT_ANALYSER */
