//
//    Beam Position Monitor
//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "devicelabels.h"

#include <QMap>

QString deviceFullName(int serial)
{
    const QString &label = deviceLabel(serial);
    return QString("BPM %1 '%2' at %3").arg(serial, 8, 16, QChar('0')).arg(label).arg(devicePlacement(label));
}

QString deviceLabel(quint64 serial)
{
    QMap<quint64, QString> label;
    label[0x0168FB3C] =  "01";//"M2";
    //    label[0x01690D03] =  "M1";
    label[0x01690DB4] =  "02";
    label[0x0168CD21] =  "03";
    label[0x016912DF] =  "04";
    label[0x01691E0A] =  "05";
    label[0x0168C827] =  "06";
    label[0x01690F26] =  "07";
    label[0x01691DA9] =  "08";
    label[0x0168E608] =  "09";
    label[0x0169117D] =  "10";
    label[0x01691A4C] =  "11";
    label[0x01690BFE] =  "12";
    label[0x0169396D] =  "13";
    label[0x0168D39F] =  "14";
    label[0x0168CAED] =  "15";
    label[0x01691D7F] =  "16";
    label[0x01693E94] =  "17";
    label[0x01692625] =  "18";
    label[0x0168DA83] =  "19";
    label[0x01693678] =  "20";
    label[0x0168D153] =  "21";
    label[0x0168E576] =  "22";
    label[0x016927BA] =  "23";
    label[0x01691FD2] =  "1FD2";
    if (label.contains(serial))
        return label[serial];
    else
        return QString("ID:%1").arg(serial, 0, 16, QChar('0'));
}

QString devicePlacement(const QString &label)
{
    QMap<QString, QString> m;
    m["01"] = "7ПЭ2";
    m["02"] = "7ПЭ3";
    m["03"] = "7ПЭ4";
    m["04"] = "8ПЭ2";
    m["05"] = "8ПЭ3";
    m["06"] = "8ПЭ4";
    m["07"] = "1ПЭ2";
    m["08"] = "1ПЭ4";
    m["09"] = "2ПЭ2";
    m["10"] = "2ПЭ4";
    m["11"] = "3ПЭ2";
    m["12"] = "3ПЭ3";
    m["13"] = "3ПЭ4";
    m["14"] = "4ПЭ2";
    m["15"] = "4ПЭ3";
    m["16"] = "4ПЭ4";
    m["17"] = "5ПЭ1";
    m["18"] = "5ПЭ2";
    m["19"] = "5ПЭ4";
    m["20"] = "6ПЭ2";
    m["21"] = "6ПЭ3";
    m["22"] = "6ПЭ4";
    m["1FD2"] = "2ПЭ1";
    if (m.contains(label))
       return m[label];
    else
        return QString("#%1").arg(label);
}

double devicePosition(const QString &place)
{
    QMap<QString, double> m;
    m["1ПЭ2"] = 0; // 251.200;
    m["1ПЭ4"] = 11.775;
    m["2ПЭ1"] = 16.600;
    m["2ПЭ2"] = 31.400;
    m["2ПЭ4"] = 43.175;
    m["3ПЭ2"] = 58.875;
    m["3ПЭ3"] = 66.725;
    m["3ПЭ4"] = 74.575;
    m["4ПЭ2"] = 90.275;
    m["4ПЭ3"] = 98.125;
    m["4ПЭ4"] = 105.975;
    m["5ПЭ1"] = 109.900;
    m["5ПЭ2"] = 125.600;
    m["5ПЭ4"] = 137.375;
    m["6ПЭ2"] = 153.075;
    m["6ПЭ3"] = 160.925;
    m["6ПЭ4"] = 168.775;
    m["7ПЭ2"] = 184.475;
    m["7ПЭ3"] = 192.325;
    m["7ПЭ4"] = 200.175;
    m["8ПЭ2"] = 215.875;
    m["8ПЭ3"] = 223.725;
    m["8ПЭ4"] = 231.575;
    if (m.contains(place))
        return m[place];
    else
        return 0;
}
