//
//    Beam Position Monitor
//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#ifndef FFTSPECTRUM_H
#define FFTSPECTRUM_H

#include <vector>
#include <iosfwd>

class FFTSpectrumElement {
public:
    FFTSpectrumElement(): freq(0), power(0), powerError(0), phase(0) {}
    FFTSpectrumElement(double f, double p, double h): freq(f), power(p), powerError(0), phase(h) {}
    FFTSpectrumElement(const FFTSpectrumElement &r): freq(r.freq), power(r.power), powerError(r.powerError), phase(r.phase) {}
    bool operator < (const FFTSpectrumElement &r) const { return power < r.power; }
    double freq;
    double power;
    double powerError;
    double phase;
};

// Magnitude peak descriptor
class FFTSpectrumPeak {
public:
    FFTSpectrumPeak(): freq(0), power(0) { }
    bool operator < (const FFTSpectrumPeak &r) const { return power < r.power; }
    double freq; // frequency
    double power; // magnitude
};

inline bool FFTSpectrumPeakLt(const FFTSpectrumPeak &l, const FFTSpectrumPeak &r) { return r<l; }

typedef std::vector<FFTSpectrumPeak> FFTSpectrumPeakVector;

class FFTSpectrum : public std::vector<FFTSpectrumElement> {
public:
    void print() const;
    double getPeak() const;
    double rms() const;
    FFTSpectrumPeakVector getPeaks(double threshold = 0.1) const;
    FFTSpectrum& operator+=(const FFTSpectrum &r) { *this = *this + r; return *this;}
    FFTSpectrum operator+(const FFTSpectrum &r) const;
    FFTSpectrum operator/(double n) const;
    FFTSpectrumElement getApproximation(double frequency) const;
};

std::ostream &operator<<(std::ostream &stream, const FFTSpectrum &r);

#endif // FFTSPECTRUM_H
