//
//    Beam Position Monitor
//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#define _USE_MATH_DEFINES
#include <cmath>

#include "VectorMaths.h"

#if 1

#include <algorithm>
#include <stdexcept>
#include "fftengine.h"

namespace VectorMaths {

    double crossCorrelation(const std::vector<double> &v1, const std::vector<double> &v2)
    {
        size_t N = std::min(v1.size(), v2.size());
        double x = 0;
        for(size_t i=0; i<N; i++)
            x += v1[i]*v2[i];
        return x;
    }

    double phaseDiff(const std::vector<double> &v1, const std::vector<double> &v2)
    {
        double ph=0;
        if (v1.empty()) throw std::runtime_error("empty v1");
        if (v2.empty()) throw std::runtime_error("empty v2");
        size_t N = std::min(v1.size(), v2.size());
        if (N == 0) throw std::runtime_error("null size");
        RFFTEngine fft(N);
        std::vector<double> fft1 = fft.fft(v1);
        std::vector<double> fft2 = fft.fft(v2);
        // 0<i<N/2,   out[N-i]=Im(Xi)
        //  for(size_t i=1; i<(N+1)/2; i++)
        //	fft2[N-i] *= -1; // complex conjugate
        //  ph = crossCorrelation(fft1, fft2);

        std::vector<double> v((N+1)/2);
        std::vector<double> vmag((N+1)/2);
        v[0] = 0; vmag[0] = (fft1[0]+fft2[0])/2;
        double norm=0, norm2=0;
        for(size_t i=1; i<(N+1)/2; i++) {
            double p1 = atan2(fft1[N-i], fft1[i]);
            double p2 = atan2(fft2[N-i], fft2[i]);
            double pd = p2-p1;
            if (pd < -M_PI) pd += 2*M_PI;
            if (pd > M_PI) pd -= 2*M_PI;
            v[i] = pd;
            vmag[i] = (sqrt(fft1[N-i]*fft1[N-i] + fft1[i]*fft1[i]) + sqrt(fft2[N-i]*fft2[N-i] + fft2[i]*fft2[i])) / 2;
            norm += vmag[i];
            norm2 += vmag[i]*vmag[i];
        }
        for(size_t i=1; i<(N+1)/2; i++) {
            ph += v[i] * vmag[i] * vmag[i];
        }
        ph /= norm2;
        /*
// Save data
  {
    std::string fileName = "v.dat";
        cerr << "Writing data "<< fileName <<endl;
        ofstream of; of.open(fileName.c_str());
    for(size_t i=0; i<v.size(); i++) {
        of << i << " " << v[i] << " " << vmag[i]/norm << endl;
      of << endl;
    }
    of.close();
  }
*/
        return ph;
    }
}
#endif
