//
//    Beam Position Monitor
//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef VECTORMATHS_H
#define VECTORMATHS_H

#include <cstddef>
#include <vector>
#include <cmath>

namespace VectorMaths {
    template<typename T>
    double calculateMean(const std::vector<T> &v)
    {
        size_t size = v.size();
        if (size==0) return 0;
        double mean=0;
        for(typename std::vector<T>::const_iterator it = v.begin(); it!=v.end(); ++it)
            mean += (*it);
        mean /= size;
        return mean;
    }

    template<typename T>
    double calculateRMS(const std::vector<T> &v)
    {
        size_t size = v.size();
        if (size==0) return 0;
        double mean=0;
        for(typename std::vector<T>::const_iterator it = v.begin(); it!=v.end(); ++it)
            mean += (*it);
        mean /= size;
        double rms=0;
        for(typename std::vector<T>::const_iterator it = v.begin(); it!=v.end(); ++it)
            rms += ((*it)-mean)*((*it)-mean);
        rms /= size;
        return sqrt(rms);
    }

    double phaseDiff(const std::vector<double> &v1, const std::vector<double> &v2);
}


#endif // VECTORMATHS_H
