//
//    Beam Position Monitor
//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef _FFTENGINE_H
#define _FFTENGINE_H

#include <vector>
#include "FFTSpectrum.h"

#include <fftw.h>
#include <rfftw.h>

class FFTPlan {
public:
    FFTPlan() {N=0;}
    virtual ~FFTPlan() {}
    virtual void resize(size_t n) = 0;
    size_t N;
};

class CFFTPlan: public FFTPlan {
public:
    CFFTPlan() {fftw_p=0; fftw_inv_p=0; in=0; out=0;}
    ~CFFTPlan() {}
    virtual void resize(size_t n);

    fftw_plan fftw_p, fftw_inv_p;
    fftw_complex *in;
    fftw_complex *out;
};

class RFFTPlan: public FFTPlan {
public:
    RFFTPlan() {fftwr_p=0; fftwr_inv_p=0; in=0; out=0;}
    ~RFFTPlan() {}
    virtual void resize(size_t n);

    rfftw_plan fftwr_p, fftwr_inv_p;
    fftw_real *in;
    fftw_real *out;
};

class FFTEngine {
public:
    FFTEngine() {wind_norm=1;N=0;}
protected:
    virtual ~FFTEngine() {}
    void update_wind_norm();
    double wind_norm;
    size_t N;
};

class CFFTEngine: public FFTEngine {
public:
    CFFTEngine() {}
    CFFTEngine(size_t n) {resize(n);}
    virtual ~CFFTEngine() {}
    void resize(size_t n) {N=n; plan.resize(N); update_wind_norm();}
protected:
    CFFTPlan plan;
};

class TimeDomainData;

class RFFTEngine: public FFTEngine {
public:
    RFFTEngine() {}
    RFFTEngine(size_t n) {resize(n);}
    virtual ~RFFTEngine() {}
    void resize(size_t n) {N=n; plan.resize(N); update_wind_norm();}
    std::vector<double> fft(const std::vector<double> &d) const;
    std::vector<double> ifft(const std::vector<double> &d) const;
    FFTSpectrum getFFTSpectrum(const TimeDomainData &d);
    //	static std::vector<FFTSpectrumPeak> getSpectrumPeaks(const FFTSpectrum &, double threshold = 0.1);
    static std::vector<double> resample(const std::vector<double> &d, double ratio, int algo, double oversample_factor);
    static std::vector<double> oversample(const std::vector<double> &d, double ratio); // less than one ratio means undersample

protected:
    RFFTPlan plan;
};

#endif /* _FFTENGINE_H */
