//
//    Beam Position Monitor
//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <QRegExp>
#include <QStringList>
#include <QDebug>

#include "appsettings.h"

AppSettings &AppSettings::merge(const AppSettings &r)
{
    foreach(QString key, r.keys()) {
        (*this)[key] = r.value(key);
    }
    return *this;
}

AppSettings AppSettings::getWithPrefix(const QString &prefix) const
{
    const QString prefixStr = prefix + "_";
    AppSettings selection;
    const AppSettings &r = *this;
    foreach(QString key, r.keys()) {
        if (key.startsWith(prefixStr)) {
            QString newKey = key.right(key.length() - prefixStr.length());
            selection[newKey] = r.value(key);
        }
    }
    return selection;
}

void AppSettings::setWithPrefix(const QString &prefix, const AppSettings &selection)
{
    foreach(QString key, selection.keys()) {
        QString newkey = prefix+"_"+key;
        (*this)[newkey] = selection.value(key);
    }
}

void AppSettings::print(const QString &title) const
{
    print(*this, title);
}

void AppSettings::print(const AppSettings &r, const QString &title)
{
    qDebug() << "AppSettings dump " << title << " begin --------------";
    foreach(QString key, r.keys()) {
        const QVariant &data = r.value(key);
        qDebug() << "  " << key << " = " << data;
    }
    qDebug() << "AppSettings dump " << title << " end ----------------";
}

QVector<double> AppSettings::getVectorOfDoubles(const QString &key) const
{
    QVector<double> v;
    const QString &data = (*this)[key].toString();
    //  qDebug(QString("AppSettings::getVectorOfDoubles('%1'): value='%2'").arg(key).arg(data));
    QRegExp listRx = QRegExp("^\\{(.+)\\}$");
    if (listRx.indexIn(data) == -1) {
        qDebug() << QString("Syntax error in parameter '%1', value '%2'").arg(key).arg(data);
        return v;
    }
    const QString &sub = listRx.cap(1);
    const QStringList &sl = sub.split(QRegExp(",\\s*"));
    //	  qDebug(QString("settings[%1]: List data '%2'").arg(key).arg(sl.join(", ")));
    foreach(QString str, sl) {
        v.push_back(str.toDouble());
    }
    return v;
}

void AppSettings::setVectorOfDoubles(const QString &key, const QVector<double> &v)
{
    QStringList sl;
    for(int i=0; i<v.size(); i++) {
        sl += QString::number(v[i], 'E', 16);
    }
    (*this)[key] = QString("{")+sl.join(", ")+"}";
}

void AppSettings::setValue(const QString &key, const QVariant &v)
{
    (*this)[key] = v;
}
